package de.ozgcloud.processor.result;

import org.springframework.stereotype.Component;

import de.ozgcloud.processor.processor.ProcessorResult;

@Component
public class AktennotizFactory {

	static final String AKTENNOTIZ_ERROR_TEXT = """
			Bei der Verarbeitung im Processor '%s' ist ein Fehler aufgetreten.
			ExceptionId: %s
			""";

	public Aktennotiz createFrom(ProcessorResult processorResult) {
		var resultBody = processorResult.getBody();
		return Aktennotiz.builder()
				.vorgangId(processorResult.getVorgangId().toString())
				.severity(resultBody.getSeverity())
				.headline(resultBody.getHeadline())
				.text(resultBody.getText())
				.build();
	}

	public Aktennotiz createErrorAktennotiz(String vorgangId, String processorName, String exceptionId) {
		return createErrorAktennotizBuilder(vorgangId)
				.text(buildErrorText(processorName, exceptionId))
				.build();
	}

	public Aktennotiz.AktennotizBuilder createErrorAktennotizBuilder(String vorgangId) {
		return Aktennotiz.builder()
				.vorgangId(vorgangId)
				.severity(Severity.ERROR)
				.headline("Fehler");
	}

	String buildErrorText(String processorName, String exceptionId) {
		return AKTENNOTIZ_ERROR_TEXT.formatted(processorName, exceptionId);
	}

}
