package de.ozgcloud.processor.result;

import jakarta.validation.Valid;

import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import de.ozgcloud.apilib.alfa.AlfaService;
import de.ozgcloud.apilib.vorgang.OzgCloudVorgangIdMapper;
import de.ozgcloud.processor.processor.ProcessorResult;
import lombok.RequiredArgsConstructor;

@Service
@RequiredArgsConstructor
@Validated
public class ResultService {

	private final AlfaService alfaService;
	private final OzgCloudVorgangIdMapper vorgangIdMapper;
	private final AktennotizFactory aktennotizFactory;

	public void processResult(@Valid ProcessorResult result) {
		if (Action.CREATE_AKTENNOTIZ == result.getAction()) {
			addAktenNotiz(aktennotizFactory.createFrom(result));
		}
	}

	public void processError(Throwable cause, String vorgangId) {
		if (cause instanceof ProcessorTechnicalException exception) {
			addAktenNotiz(
					aktennotizFactory.createErrorAktennotiz(exception.getVorgangId(), exception.getProcessorName(), exception.getExceptionId()));
		} else {
			addAktenNotiz(aktennotizFactory.createErrorAktennotizBuilder(vorgangId)
					.text("Es ist eine Fehler bei der Verarbeitung des Vorganges aufgetreten").build());
		}
	}

	void addAktenNotiz(Aktennotiz aktennotiz) {
		alfaService.addAktenNotiz(vorgangIdMapper.fromString(aktennotiz.getVorgangId()), aktennotiz.getText());
	}

}
