package de.ozgcloud.processor.processor;

import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.Optional;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.Spy;
import org.springframework.http.client.reactive.ClientHttpConnector;
import org.springframework.web.reactive.function.client.WebClient;

class ProcessorConfigurationTest {

	@Spy
	@InjectMocks
	private ProcessorConfiguration processorConfiguration;

	@Mock
	private WebClient.Builder webClienBuilder;
	@Mock
	private ClientHttpConnector clientHttpConnector;

	@Spy
	private Optional<ProxyProperties> proxyProperties = Optional.of(ProxyPropertiesTestFactory.create());

	@Nested
	class TestCreateProxy {

		@Test
		void shouldCallCreateProxyConnector() {
			processorConfiguration.webClient(webClienBuilder);

			verify(processorConfiguration).createProxyConnector(proxyProperties.get());
		}

		@Test
		void shouldAddProxyConnector() {
			doReturn(clientHttpConnector).when(processorConfiguration).createProxyConnector(any());

			processorConfiguration.webClient(webClienBuilder);

			verify(webClienBuilder).clientConnector(clientHttpConnector);
		}
	}

	@Test
	void shouldNotAddProxyConnector() {
		processorConfiguration = Mockito.spy(new ProcessorConfiguration(Optional.empty()));

		processorConfiguration.webClient(webClienBuilder);

		verify(processorConfiguration, never()).createProxyConnector(any());
		verify(webClienBuilder, never()).clientConnector(any());
	}
}