package de.ozgcloud.processor.processor;

import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.context.ApplicationEventPublisher;

import de.ozgcloud.apilib.vorgang.OzgCloudVorgangTestFactory;
import de.ozgcloud.command.CommandCreatedEvent;
import de.ozgcloud.command.CommandTestFactory;
import de.ozgcloud.command.VorgangCreatedEvent;
import de.ozgcloud.common.test.ITCase;
import de.ozgcloud.processor.result.ResultService;
import de.ozgcloud.processor.vorgang.VorgangId;
import de.ozgcloud.processor.vorgang.VorgangService;
import de.ozgcloud.processor.vorgang.VorgangTestFactory;

@ITCase
class ProcessorEventListenerITCase {

	@MockBean
	private ResultService resultService;
	@MockBean
	private VorgangService service;
	@MockBean
	private ProcessorService processorService;

	@Autowired
	private ApplicationEventPublisher publisher;

	@Test
	void shouldLoadVorgang() {
		when(service.getVorgang(any())).thenReturn(VorgangTestFactory.create());
		var event = new VorgangCreatedEvent(OzgCloudVorgangTestFactory.ID.toString());

		publisher.publishEvent(event);

		verify(service).getVorgang(VorgangId.from(OzgCloudVorgangTestFactory.ID.toString()));
	}

	@DisplayName("On command listener")
	@Nested
	class TestOnCommandListener {

		@Test
		void shouldTriggerForExecuteProcessor() {
			var command = CommandTestFactory.createBuilder().order(ProcessorEventListener.PROCESS_VORGANG_ORDER).build();

			publisher.publishEvent(new CommandCreatedEvent(command));

			verify(service).getVorgang(VorgangId.from(CommandTestFactory.VORGANG_ID));
		}
	}
}
