package de.ozgcloud.processor.processor;

import static org.assertj.core.api.Assertions.*;

import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.test.util.ReflectionTestUtils;

import de.ozgcloud.common.test.ITCase;
import lombok.SneakyThrows;

@ITCase
class ProcessorPropertiesITCase {

	@Autowired
	private ProcessorService service;

	@Test
	void shouldHaveTwoProperties() {
		var props = getProperties();

		assertThat(props.getProcessors()).hasSize(2);
	}

	@Test
	@SneakyThrows
	void shouldHaveFirstAddress() {
		var processors = getProperties().getProcessors();

		assertThat(processors.get(0).getAddress()).isEqualTo("http://test1");
	}

	@Test
	void shouldHaveFormIds() {
		var forms = getProperties().getProcessors().get(0).getForms();

		assertThat(forms.stream().map(ProcessorProperties.Form::getFormId)).contains("ID1", "ID2");
	}

	@Test
	void shouldHaveFormEngineName() {
		var forms = getProperties().getProcessors().get(0).getForms();

		assertThat(forms.stream().map(ProcessorProperties.Form::getFormEngineName)).containsOnly("formEngineTest1");
	}

	private ProcessorProperties getProperties() {
		return (ProcessorProperties) ReflectionTestUtils.getField(service, "properties");
	}

}
