package de.ozgcloud.processor.result;

import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import de.ozgcloud.apilib.alfa.AlfaService;
import de.ozgcloud.apilib.vorgang.OzgCloudVorgangId;
import de.ozgcloud.apilib.vorgang.OzgCloudVorgangIdMapper;
import de.ozgcloud.processor.processor.ProcessorResult;
import de.ozgcloud.processor.processor.ProcessorResultTestFactory;
import de.ozgcloud.processor.processor.ProcessorTestFactory;
import de.ozgcloud.processor.vorgang.VorgangTestFactory;

class ResultServiceTest {

	@Spy
	@InjectMocks
	private ResultService resultService;

	@Mock
	private AlfaService alfaService;
	@Mock
	private OzgCloudVorgangIdMapper vorgangIdMapper;
	@Mock
	private AktennotizFactory aktennotizFactory;

	private Aktennotiz aktennotiz = AktennotizTestFactory.create();

	@Nested
	class TestProcessResult {

		private ProcessorResult processorResult = ProcessorResultTestFactory.create();

		@Test
		void shouldCallCreateAktennotiz() {
			when(aktennotizFactory.createFrom(any())).thenReturn(aktennotiz);

			resultService.processResult(processorResult);

			verify(aktennotizFactory).createFrom(processorResult);
		}

		@Test
		void shouldCallAddAktenNotiz() {
			when(aktennotizFactory.createFrom(any())).thenReturn(aktennotiz);

			resultService.processResult(processorResult);

			verify(resultService).addAktenNotiz(aktennotiz);
		}
	}

	@Nested
	class TestProcessError {

		private static final String EXCEPTION_ID = "exceptionId";

		@Mock
		private ProcessorTechnicalException exception;

		@BeforeEach
		void setup() {
			when(exception.getVorgangId()).thenReturn(VorgangTestFactory.ID.toString());
			when(exception.getExceptionId()).thenReturn(EXCEPTION_ID);
			when(aktennotizFactory.createErrorAktennotiz(any(), any(), any())).thenReturn(aktennotiz);
		}

		@Test
		void shouldCallCreateErrorAktennotiz() {
			when(exception.getProcessorName()).thenReturn(ProcessorTestFactory.PROCESSOR_NAME);
			when(exception.getExceptionId()).thenReturn(EXCEPTION_ID);

			resultService.processError(exception, VorgangTestFactory.ID.toString());

			verify(aktennotizFactory).createErrorAktennotiz(VorgangTestFactory.ID.toString(), ProcessorTestFactory.PROCESSOR_NAME, EXCEPTION_ID);
		}

		@Test
		void shouldCallAddAktenNotiz() {
			resultService.processError(exception, VorgangTestFactory.ID.toString());

			verify(resultService).addAktenNotiz(aktennotiz);
		}
	}

	@Nested
	class TestAddAktenNotiz {

		@Mock
		private OzgCloudVorgangId ozgCloudVorgangId;

		@Test
		void shouldCallMapper() {
			resultService.addAktenNotiz(aktennotiz);

			verify(vorgangIdMapper).fromString(aktennotiz.getVorgangId());
		}

		@Test
		void shouldCallAlfaService() {
			when(vorgangIdMapper.fromString(any())).thenReturn(ozgCloudVorgangId);

			resultService.addAktenNotiz(aktennotiz);

			verify(alfaService).addAktenNotiz(ozgCloudVorgangId, aktennotiz.getText());
		}
	}
}