package de.ozgcloud.processor.vorgang;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;

import de.ozgcloud.apilib.vorgang.OzgCloudVorgang;
import de.ozgcloud.apilib.vorgang.OzgCloudVorgangService;
import de.ozgcloud.apilib.vorgang.OzgCloudVorgangTestFactory;

class VorgangServiceTest {

	@InjectMocks
	private VorgangService service;

	@Mock
	private OzgCloudVorgangService ozgCloudService;
	@Mock
	private ProcessorVorgangMapper mapper;

	@Nested
	class TestGetVorgang {

		private OzgCloudVorgang ozgCloudVorgang = OzgCloudVorgangTestFactory.create();
		private Vorgang vorgang = VorgangTestFactory.create();

		@BeforeEach
		void init() {
			when(ozgCloudService.getById(any())).thenReturn(ozgCloudVorgang);
			when(mapper.mapVorgang(any())).thenReturn(vorgang);
		}

		@Test
		void shouldCallOzgCloudService() {
			service.getVorgang(VorgangTestFactory.ID);

			verify(ozgCloudService).getById(OzgCloudVorgangTestFactory.ID);
		}

		@Test
		void shouldCallMapper() {
			service.getVorgang(VorgangTestFactory.ID);

			verify(mapper).mapVorgang(ozgCloudVorgang);
		}

		@Test
		void shouldReturnMappedVorgang() {
			var result = service.getVorgang(VorgangTestFactory.ID);

			assertThat(result).isSameAs(vorgang);
		}
	}
}
